from fastapi import FastAPI
import pandas as pd
import os
import json
import logging
from typing import Dict
from grassat.lingra import *
from grassat.prepare_data import *
from grassat.model.field_data import *

app = FastAPI()

def setup_logging(debug: bool):
    level = logging.INFO
    if debug:
        level = logging.DEBUG
    logging.basicConfig(level=level, format='%(asctime)s %(levelname)-8s %(message)s')

@app.post("/model")
def run_model(request_body: Dict, debug: bool = False):
    # Get the current working directory to access input data
    current_path = os.getcwd()
    
    field_data: FieldData = FieldData.from_request(request_body['grassatRequest'])
    
    # Set output dir
    outdir = os.path.join(current_path,"output")
    os.makedirs(outdir, exist_ok=True)

    # Get necessary data in required formats
    irrigations_df = getIrrigations(field_data.irrigations)
    fertilisations_df = getFertilisations(field_data.fertilisations)
    harvestDays = getHarvestDays(field_data.harvests)
    weather_df = getMeteoData(field_data)
    soil_df = getSoilData(field_data.geometry)   
 
    # Cut harvest, fertilisation and irrigation data to length of weather data
    harvestDays = [x for x in harvestDays if x <= weather_df["DOY"].max()]
    if not fertilisations_df.empty:
        fertilisations_df=fertilisations_df[fertilisations_df['Dates'] <= weather_df["DOY"].max()]    
    if not irrigations_df.empty:
        irrigations_df=irrigations_df[irrigations_df['Dates'] <= weather_df["DOY"].max()]

    # Run the lingra model and plot output and store in outdir
    lingra_data = lingra(weather_df, harvestDays, fertilisations_df, irrigations_df, soil_df)
    response = lingra_wig_response(lingra_data)
    logging.info(f'Output data: {lingra_data}')
    logging.info(f'Output data json: {response}')
    return json.loads(response)

